import { Buffer } from "buffer";
import { v4 as uuidv4 } from "uuid";

const wssProtocol = window.location.protocol === "https:" ? "wss://" : "ws://";

export const Constants = {
  API_ADDRESS: `${window.location.protocol}//${window.location.hostname}/api/v1`,
  WS_ADDRESS: `${wssProtocol}${window.location.hostname}/apiws`,
  STATIC_CONTENT_ADDRESS: `${window.location.protocol}//${window.location.hostname}/api/static`,
  ROUTE_PATHS: {
    LESSIONS: {
      ROOT: "/lessons",
      PAGE: "/lessons/:lessonId",
      PAGE_EDITOR: "/lessons/:lessonId/editor",
    },
    ORGANIZATION_TEAM: "/team",
    ORGANIZATION_TEAM_CREATE_USER: "/team/create-user",
    ORGANIZATION_ROLES: "/roles",
    ORGANIZATION_SETTINGS: "/organization",
    ACCOUNT_SETTINGS: "/account",
    WHATS_NEW: "/whats-new",
    SUGGEST_FEATURE: "/suggest-feature",
    CONTACT_SUPPORT: "/contact-support",
  },
  STYLES: {
    BLACK: "#000",
  },
  MAX_IMAGE_SIZE: 25 * 1024 * 1024, // 25MB
  ACCEPTED_IMAGE_FILE_TYPES: [
    "image/png",
    "image/jpeg",
    "image/jpg",
    "image/webp",
  ],
};

// used for sideMenu
export const BreakpointLgWidth = 992;

export function GetUuid() {
  return uuidv4();
}

export function getImageUrl(imageName: string) {
  return `${Constants.STATIC_CONTENT_ADDRESS}/${imageName}`;
}

// needed for a user who uses multiple tabs in the browser
// with the same session id because otherwise the last browser
// tab would subscribe to the topic and the other tabs would
// not receive any messages
// used for topic subscription
export const BrowserTabSession = GetUuid();

export function getUserSessionFromLocalStorage() {
  return localStorage.getItem("session");
}

export function EncodeStringToBase64(value: string) {
  return Buffer.from(value).toString("base64");
}

export function DecodedBase64ToString(value: string) {
  return Buffer.from(value, "base64").toString();
}

export function handleLogout() {
  localStorage.removeItem("session");
  window.location.href = "/";
}

export const myFetchContentType = {
  JSON: 0,
  FORM_DATA: 1,
};

type Method = "GET" | "POST" | "PUT" | "DELETE" | "PATCH";

interface MyFetchOptions<TRequest = any, TResponse = any> {
  url?: string;
  method?: Method;
  body?: TRequest | null;
  headers?: Record<string, string>;
  contentType?: "JSON" | "FORM_DATA";
  fetchUrl?: string;
  ignoreUnauthorized?: boolean;
  notificationApi?: any; // Passen Sie dies je nach Bedarf an
  t?: any; // Passen Sie dies je nach Bedarf an
}

export function myFetch<TRequest = any, TResponse = any>({
  url = "",
  method = "GET",
  body = null,
  headers = {},
  contentType = "JSON",
  fetchUrl = Constants.API_ADDRESS,
  ignoreUnauthorized = false,
  notificationApi = null,
  t = null,
}: MyFetchOptions<TRequest, TResponse>): Promise<TResponse> {
  const getContentType = () => {
    if (contentType === "JSON") return "application/json";

    return "multipart/form-data";
  };

  const getBody = () => {
    if (!body) return null;

    if (contentType === "JSON") return JSON.stringify(body);

    return body;
  };

  // abort fetch if it takes to long
  const controller = new AbortController();
  const signal = controller.signal;

  setTimeout(() => controller.abort(), 30000); // 30 seconds

  const requestOptions = {
    method: method,
    headers: {
      "X-Authorization": getUserSessionFromLocalStorage() || "",
      "Content-Type": getContentType(),
      ...headers,
    },
    body: getBody(),
    signal: signal,
  };

  if (fetchUrl === "") {
    fetchUrl = Constants.API_ADDRESS;
  }

  return fetch(`${fetchUrl}${url}`, requestOptions as RequestInit)
    .then(async (response) => {
      // if status is not in range 200-299
      if (!response.ok) {
        if (!ignoreUnauthorized && response.status === 401) {
          console.error("Unauthorized");
          // TODO: check here
          //setUserSessionToLocalStorage("");
          //window.location.href = "/";
        }

        throw response.status;
      }

      // check if response is json
      if (response.headers.get("content-type")?.includes("application/json")) {
        return response.json();
      }

      return response.text();
    })
    .catch(async (error) => {
      console.error("Error", error);

      // ignore errors here as they are handled in the components
      if (error === 400) throw error;

      if (error === 401) {
        if (ignoreUnauthorized) {
          throw error;
        }

        handleLogout();
        return;
      }

      // show error notification for all other errors

      if (notificationApi !== null && t !== null) {
        if (error === 500) {
          /* notificationApi["error"]({
            message: t("common.request.failed.title"),
            description: t("common.request.failed.description"),
          }); */
        } else {
          // other errors
          /* notificationApi["error"]({
            message: t("common.request.failedInternetProblem.title"),
            description: t("common.request.failedInternetProblem.description"),
          }); */
        }
      }

      throw error;
    });
}
